@@ -1,8 +1,5 @@
 package ExtUtils::InstallPaths;
-{
-  $ExtUtils::InstallPaths::VERSION = '0.008';
-}
-
+$ExtUtils::InstallPaths::VERSION = '0.011';
 use 5.006;
 use strict;
 use warnings;
@@ -25,37 +22,40 @@ my %defaults = (
 	module_name     => undef,
 	destdir         => undef,
 	install_path    => sub { {} },
-	install_sets    => sub { _default_install_sets($_[0]) },
-	original_prefix => sub { _default_original_prefix($_[0]) },
-	install_base_relpaths => sub { _default_base_relpaths($_[0]) },
-	prefix_relpaths => sub { _default_prefix_relpaths($_[0]) },
+	install_sets    => \&_default_install_sets,
+	original_prefix => \&_default_original_prefix,
+	install_base_relpaths => \&_default_base_relpaths,
+	prefix_relpaths => \&_default_prefix_relpaths,
 );
 
-sub _merger {
-	my $name = shift;
+sub _merge_shallow {
+	my ($name, $filter) = @_;
 	return sub {
 		my ($override, $config) = @_;
 		my $defaults = $defaults{$name}->($config);
-
-		my %new_opts = %$override;
-		while (my ($key, $val) = each %$defaults) {
-			if (exists $override->{$key}) {
-				if (ref($val) eq 'HASH') {
-					while (my ($k, $v) = each %$val) {
-					   $new_opts{$key}{$k} = $v unless exists $override->{$key}{$k};
-					}
-				}
-			} else {
-				$new_opts{$key} = $val
-			}
-		}
-
-		return \%new_opts;
+		$filter->($_) for grep $filter, values %$override;
+		return { %$defaults, %$override };
 	}
 }
 
+sub _merge_deep {
+	my ($name, $filter) = @_;
+	return sub {
+		my ($override, $config) = @_;
+		my $defaults = $defaults{$name}->($config);
+		my $pair_for = sub {
+			my $key = shift;
+			my %override = %{ $override->{$key} || {} };
+			$filter && $filter->($_) for values %override;
+			return $key => { %{ $defaults->{$key} }, %override };
+		};
+		return { map { $pair_for->($_) } keys %$defaults };
+	}
+}
 
 my %allowed_installdir = map { $_ => 1 } qw/core site vendor/;
+my $must_be_relative = sub { Carp::croak('Value must be a relative path') if File::Spec->file_name_is_absolute($_[0]) };
+my %deep_filter = map { $_ => $must_be_relative } qw/install_base_relpaths prefix_relpaths/;
 my %filter = (
 	installdirs => sub {
 		my $value = shift;
@@ -63,9 +63,21 @@ my %filter = (
 		Carp::croak('installdirs must be one of "core", "site", or "vendor"') if not $allowed_installdir{$value};
 		return $value;
 	},
-	map { $_ => _merger($_) } qw/install_sets original_prefix install_base_relpaths prefix_relpaths/,
+	(map { $_ => _merge_shallow($_, $deep_filter{$_}) } qw/original_prefix install_base_relpaths/),
+	(map { $_ => _merge_deep($_, $deep_filter{$_}) } qw/install_sets prefix_relpaths/),
 );
 
+sub new {
+	my ($class, %args) = @_;
+	my $config = $args{config} || ExtUtils::Config->new;
+	my %self = (
+		config => $config,
+		map { $_ => exists $args{$_} ? $filter{$_} ? $filter{$_}->($args{$_}, $config) : $args{$_} : ref $defaults{$_} ? $defaults{$_}->($config) : $defaults{$_} } keys %defaults,
+	);
+	$self{module_name} ||= do { my $module_name = $self{dist_name}; $module_name =~ s/-/::/g; $module_name } if defined $self{dist_name};
+	return bless \%self, $class;
+}
+
 for my $attribute (keys %defaults) {
 	no strict qw/refs/;
 	*{$attribute} = $hash_accessors{$attribute} ? 
@@ -87,58 +99,23 @@ for my $attribute (keys %defaults) {
 	};
 }
 
-sub new {
-	my ($class, %args) = @_;
-	my $config = $args{config} || ExtUtils::Config->new;
-	my %self = (
-		config => $config,
-		map { $_ => exists $args{$_} ? $filter{$_} ? $filter{$_}->($args{$_}, $config) : $args{$_} : ref $defaults{$_} ? $defaults{$_}->($config) : $defaults{$_} } keys %defaults,
-	);
-	$self{module_name} ||= do { my $module_name = $self{dist_name}; $module_name =~ s/-/::/g; $module_name } if defined $self{dist_name};
-	return bless \%self, $class;
-}
+my $script = $] > 5.008000 ? 'script' : 'bin';
+my @install_sets_keys = qw/lib arch bin script bindoc libdoc binhtml libhtml/;
+my @install_sets_tail = ('bin', $script, qw/man1dir man3dir html1dir html3dir/);
+my %install_sets_values = (
+	core   => [ qw/privlib archlib /, @install_sets_tail ],
+	site   => [ map { "site$_" } qw/lib arch/, @install_sets_tail ],
+	vendor => [ map { "vendor$_" } qw/lib arch/, @install_sets_tail ],
+);
 
 sub _default_install_sets {
 	my $c = shift;
 
-	my $bindoc  = $c->get('installman1dir');
-	my $libdoc  = $c->get('installman3dir');
-
-	my $binhtml = $c->get('installhtml1dir') || $c->get('installhtmldir');
-	my $libhtml = $c->get('installhtml3dir') || $c->get('installhtmldir');
-
-	return {
-		core   => {
-			lib     => $c->get('installprivlib'),
-			arch    => $c->get('installarchlib'),
-			bin     => $c->get('installbin'),
-			script  => $c->get('installscript'),
-			bindoc  => $bindoc,
-			libdoc  => $libdoc,
-			binhtml => $binhtml,
-			libhtml => $libhtml,
-		},
-		site   => {
-			lib     => $c->get('installsitelib'),
-			arch    => $c->get('installsitearch'),
-			bin     => $c->get('installsitebin') || $c->get('installbin'),
-			script  => $c->get('installsitescript') || $c->get('installsitebin') || $c->get('installscript'),
-			bindoc  => $c->get('installsiteman1dir') || $bindoc,
-			libdoc  => $c->get('installsiteman3dir') || $libdoc,
-			binhtml => $c->get('installsitehtml1dir') || $binhtml,
-			libhtml => $c->get('installsitehtml3dir') || $libhtml,
-		},
-		vendor => {
-			lib     => $c->get('installvendorlib'),
-			arch    => $c->get('installvendorarch'),
-			bin     => $c->get('installvendorbin') || $c->get('installbin'),
-			script  => $c->get('installvendorscript') || $c->get('installvendorbin') || $c->get('installscript'),
-			bindoc  => $c->get('installvendorman1dir') || $bindoc,
-			libdoc  => $c->get('installvendorman3dir') || $libdoc,
-			binhtml => $c->get('installvendorhtml1dir') || $binhtml,
-			libhtml => $c->get('installvendorhtml3dir') || $libhtml,
-		},
-	};
+	my %ret;
+	for my $installdir (qw/core site vendor/) {
+		@{$ret{$installdir}}{@install_sets_keys} = map { $c->get("install$_") } @{ $install_sets_values{$installdir} };
+	}
+	return \%ret;
 }
 
 sub _default_base_relpaths {
@@ -155,6 +132,15 @@ sub _default_base_relpaths {
 	};
 }
 
+my %common_prefix_relpaths = (
+	bin        => ['bin'],
+	script     => ['bin'],
+	bindoc     => ['man', 'man1'],
+	libdoc     => ['man', 'man3'],
+	binhtml    => ['html'],
+	libhtml    => ['html'],
+);
+
 sub _default_prefix_relpaths {
 	my $c = shift;
 
@@ -166,32 +152,17 @@ sub _default_prefix_relpaths {
 		core => {
 			lib        => [@libstyle],
 			arch       => [@libstyle, $version, $arch],
-			bin        => ['bin'],
-			script     => ['bin'],
-			bindoc     => ['man', 'man1'],
-			libdoc     => ['man', 'man3'],
-			binhtml    => ['html'],
-			libhtml    => ['html'],
+			%common_prefix_relpaths,
 		},
 		vendor => {
 			lib        => [@libstyle],
 			arch       => [@libstyle, $version, $arch],
-			bin        => ['bin'],
-			script     => ['bin'],
-			bindoc     => ['man', 'man1'],
-			libdoc     => ['man', 'man3'],
-			binhtml    => ['html'],
-			libhtml    => ['html'],
+			%common_prefix_relpaths,
 		},
 		site => {
 			lib        => [@libstyle, 'site_perl'],
 			arch       => [@libstyle, 'site_perl', $version, $arch],
-			bin        => ['bin'],
-			script     => ['bin'],
-			bindoc     => ['man', 'man1'],
-			libdoc     => ['man', 'man3'],
-			binhtml    => ['html'],
-			libhtml    => ['html'],
+			%common_prefix_relpaths,
 		},
 	};
 }
@@ -200,11 +171,10 @@ sub _default_original_prefix {
 	my $c = shift;
 
 	my %ret = (
-		core   => $c->get('installprefixexp') || $c->get('installprefix') || $c->get('prefixexp') || $c->get('prefix') || '',
+		core   => $c->get('installprefixexp'),
 		site   => $c->get('siteprefixexp'),
 		vendor => $c->get('usevendorprefix') ? $c->get('vendorprefixexp') : '',
 	);
-	$ret{site} ||= $ret{core};
 
 	return \%ret;
 }
@@ -226,8 +196,6 @@ sub is_default_installable {
 	return $installable ? 1 : 0;
 }
 
-#Carp::croak('Value must be a relative path') if File::Spec::Unix->file_name_is_absolute($value);
-
 sub _prefixify_default {
 	my $self = shift;
 	my $type = shift;
@@ -425,17 +393,19 @@ sub install_map {
 
 # ABSTRACT: Build.PL install path logic made easy
 
-
 __END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 ExtUtils::InstallPaths - Build.PL install path logic made easy
 
 =head1 VERSION
 
-version 0.008
+version 0.011
 
 =head1 SYNOPSIS
 
@@ -658,4 +628,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
